/*
  ==============================================================================

   This file is part of the Plugin Consultant - VST API Logger
   Copyright 2005 by Ofer Shem Tov.

  ------------------------------------------------------------------------------

   Plugin Consultant can be redistributed and/or modified under the terms of the
   GNU General Public License, as published by the Free Software Foundation;
   either version 2 of the License, or (at your option) any later version.

   Plugin Consultant is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Plugin Consultant; if not, visit www.gnu.org/licenses or write to the
   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
   Boston, MA 02111-1307 USA

  ==============================================================================
*/

#ifdef _MSC_VER
#pragma warning (disable : 4996)
#endif

#include <aeffectx.h>
#include <juce.h>
#include "PCLogger.h"

extern AEffect proxyEffect;
extern PCLogger pcLogger;
extern File lastDir;
extern void copyAEffect();
extern File dllPath;

static audioMasterCallback vstAudioMaster = NULL;
static String path;

long VSTCALLBACK proxyAudioMaster(AEffect *effect, long opcode, long index, long value, void *ptr, float opt);

typedef AEffect* (/*__stdcall*/* vstPluginMain)(audioMasterCallback);

//-----------------------------------------------------------------------------------------
#ifdef WIN32

static void* pluginHandle = NULL;

vstPluginMain getVstPluginMain()
{
	// load the plugin DLL
	pluginHandle = Process::loadDynamicLibrary( path );

	if ( pluginHandle == NULL )
	{
		pcLogger.log("Failed to load " + path );
		return NULL;
	}

	pcLogger.log("Loaded " + path );

	return (vstPluginMain)Process::getProcedureEntryPoint(pluginHandle, "main");
}

bool freeVstPlugin()
{
	if ( pluginHandle == NULL )
		return false;
		
	Process::freeDynamicLibrary( pluginHandle );
	pluginHandle = NULL;
	return true;
}

#endif

//-----------------------------------------------------------------------------------------
#if MAC

CFBundleRef vstBundle = NULL;
short bundleRes;

vstPluginMain getVstPluginMain()
{
	CFStringRef vstBundlePath = CFStringCreateWithCString( kCFAllocatorDefault, path, kCFStringEncodingMacRoman );
	CFURLRef vstBundleURL = CFURLCreateWithFileSystemPath(
							kCFAllocatorDefault, 
							vstBundlePath,
							kCFURLPOSIXPathStyle,
							true );
							
	vstBundle =  CFBundleCreate( kCFAllocatorDefault, vstBundleURL );
	CFRelease( vstBundlePath );
	CFRelease( vstBundleURL );
	
	if ( vstBundle == NULL )
	{
		pcLogger.log("Failed to load " + path );
		return NULL;
	}
	
	bundleRes = CFBundleOpenBundleResourceMap( vstBundle );
	
	return  (vstPluginMain) CFBundleGetFunctionPointerForName( vstBundle, CFSTR("main_macho") );
}

bool freeVstPlugin()
{
	if ( vstBundle == NULL )
		return false;
		
	CFBundleCloseBundleResourceMap( vstBundle, bundleRes );
	CFBundleUnloadExecutable( vstBundle );
	CFRelease( vstBundle );
	vstBundle = NULL;
	return true;
}

#endif

//-----------------------------------------------------------------------------------------
AEffect* loadVstPlugin(audioMasterCallback audioMaster)
{
	// remember original audio master
	vstAudioMaster = audioMaster;

	if ( !lastDir.isDirectory() )
		lastDir = SystemStats::getUserHomeDirectory();
		
#if WIN32
char vstFilter[] = "*.dll";
#endif
#if MAC
char vstFilter[] = "*.vst";
#endif 

	FileChooser myChooser (dllPath.getFileNameWithoutExtension() +
							" - Please select the plugin to log...", lastDir, vstFilter);

	// choose plugin to load
	if (myChooser.browseForFileToOpen())
	{
		File pluginFile	(myChooser.getResult());
		lastDir = pluginFile.getParentDirectory();
		path = pluginFile.getFullPathName();

		// set logger name to this plug name
		pcLogger.setPlugName( pluginFile.getFileName() );
	}
	else
	{
		pcLogger.log( T("No plugin was selected") );
		return NULL;
	}

	vstPluginMain getNewPlugInstance = getVstPluginMain();

	AEffect* newPlugin = NULL;

	if( getNewPlugInstance != NULL )
	{
#ifdef MAC
		short res = CurResFile();
		UseResFile( bundleRes );
#endif // MAC
	
		// get the plugin effect object and give it our proxy audio master
		newPlugin = getNewPlugInstance(proxyAudioMaster);

#ifdef MAC
		UseResFile( res );
#endif // MAC
			
		// make sure it is a VST plugin
		if( newPlugin != NULL )
			if( newPlugin->magic!=kEffectMagic )
			{
				newPlugin = NULL;	// its not a VST
				pcLogger.log( T("Wrong Magic, probably not a VST plugin") );
			}
	}
	else
		pcLogger.log( T("Failed to get VST main function") );


	// if file loaded but plugin failed then unload the file
	if ( newPlugin == NULL )
	{
		freeVstPlugin();
		pcLogger.log("Unloaded " + path );
	}

	return newPlugin;
}

//-----------------------------------------------------------------------------------------
void	unloadPlugin()
{
		if ( freeVstPlugin() )
			pcLogger.log("Unloaded " + path );
}

//-----------------------------------------------------------------------------------------
long VSTCALLBACK proxyAudioMaster(AEffect *effect, long opcode, long index, long value, void *ptr, float opt)
{
	// the effect might changed its AEffect structure before calling the host (e.g. audioMasterIOChanged)
	void copyAEffect();

	pcLogger.logVstAudioMaster(opcode,index,value,ptr,opt);
	long retval = vstAudioMaster(&proxyEffect,opcode,index,value,ptr,opt);
	pcLogger.logVstAudioMasterReturn( opcode, ptr, retval );
	return retval;
}
